/*

    ThermostatQA - test framework for Thermostat Monitoring Tool

    Copyright 2013 Red Hat, Inc.

This file is part of ThermostatQA

ThermostatQA is distributed under the GNU General Public License,
version 2 or any later version (with a special exception described
below, commonly known as the "Classpath Exception").

A copy of GNU General Public License (GPL) is included in this
distribution, in the file COPYING.

Linking ThermostatQA code with other modules is making a combined work
based on ThermostatQA.  Thus, the terms and conditions of the GPL
cover the whole combination.

As a special exception, the copyright holders of ThermostatQA give you
permission to link this code with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on ThermostatQA code.  If you modify ThermostatQA, you may
extend this exception to your version of the software, but you are
not obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.thermostat.qa.testsuites;



import java.io.File;
import java.io.IOException;
import java.util.List;



import org.thermostat.qa.annotations.TestType;
import org.thermostat.qa.annotations.TestTypes;
import org.thermostat.qa.framework.Assert;
import org.thermostat.qa.framework.ThermostatTest;


/**
 * Class SmokeTests contains basic smoke tests for testing if the thermostat
 * executable works properly. Thermostat executable should
 * exist as a file, be readable, executable, exit with the right
 * value, and have nonempty output. 
 * 
 */
@TestType(TestTypes.OTHER_TEST)
public class SmokeTests extends ThermostatTest
{
    @Override
    protected void setUp()
    {
        // empty as expected
    }

    @Override
    protected void tearDown()
    {
        // empty as expected
    }

    private File getThermostatExecutableAsFile()
    {
        return new File(this.configuration.getThermostatExecutable());
    }

    /**
     * Method testCheckPathToThermostatExecutable
     * tests if the string describing path to thermostat is not null,
     * and if the file thermostat really exists.
     */
    public void testCheckPathToThermostatExecutable()
    {
        File f = getThermostatExecutableAsFile();
        Assert.assertNotNull(f, this.configuration.getThermostatExecutable() + " does not exist");
        Assert.assertTrue(f.exists(), this.configuration.getThermostatExecutable() + " does not exist");
    }

    /**
     * Method testCheckIfThermostatExecutableCanBeRead
     * tests if thermostat executable is readable by the application.
     */
    public void testCheckIfThermostatExecutableCanBeRead()
    {
        File f = getThermostatExecutableAsFile();
        Assert.assertTrue(f.canRead(), this.configuration.getThermostatExecutable() + " is not readable");
    }

    /**
     * Method testCheckIfThermostatExecutableCanBeExecuted
     * tests if application is allowed to execute thermostat executable.
     */
    public void testCheckIfThermostatExecutableCanBeExecuted()
    {
        File f = getThermostatExecutableAsFile();
        Assert.assertTrue(f.canExecute(), this.configuration.getThermostatExecutable() + " is not executable");
    }

    /**
     * Method testTryToRunThermostatExecutable
     * runs (exec) the thermostat executable, no assert, problems can be
     * errors and exceptions.
     * 
     * @throws IOException
     */
    public void testTryToRunThermostatExecutable() throws IOException
    {
        Process process = runThermostatInNewProcess();
        waitForExternalProcess(process);
    }

    /**
     * Method testCheckExitValue
     * tries to run thermostat executable and tests if the exit value
     * is the same as expected exit value.
     * 
     * @throws IOException
     */
    public void testCheckExitValue() throws IOException
    {
        Process process = runThermostatInNewProcess();
        waitForExternalProcess(process);
        int exitValue = process.exitValue();
        Assert.assertEquals(exitValue, EXPECTED_EXIT_VALUE, "Bad thermostat exit value " + exitValue);
    }

    /**
     * Method testCheckStandardMessage
     * runs thermostat in a new process, tests if the process output
     * is not null and if the output size is nonzero.
     *  
     * @throws IOException
     */
    public void testCheckStandardMessage() throws IOException
    {
        Process process = runThermostatInNewProcess();
        List<String> processOutput = readProcessOutput(process);
        Assert.assertNotNull(processOutput, "no output from process");
        Assert.assertTrue(processOutput.size() > 0, "no output from process");
    }

    public static void main(String[] args)
    {
        new SmokeTests().doTests(args);
    }
}
